%% DC Motor: Closed-Loop Proportional Control

%% Model Parameters

% Nominal ODE Coefficients
a0Nom = 0.94;                      % ODE coefficient, 1/sec
b0Nom = 766.8;                     % ODE coefficient, rad/(sec^2 V)

% Nominal Motor Model
a0 = a0Nom;
b0 = b0Nom;
G = tf(b0,[1 a0]);

% Maximum Operating Voltage, V
umax = 3;

%% Simulations on Nominal Model
% The inputs/outputs of the proportional controller are:
%  Inputs:
%      1) Desired rotational speed r, rad/sec
%      2) Actual rotational speed y, rad/sec
%  Output: Motor voltage u, V
% The proportional controller computes the error e=r-y and then
% select the motor voltage proportional to this error, i.e. u=Kp*e.


% Simulation and plot settings
ulim = inf;     % Limit on max motor voltage for simulation, V
d = -0.2;        % Input disturbance, V
Tf = 4;         % Final simulation time, sec
co = get(groot,'defaultAxesColorOrder');

% Simulate with several proportional gains
figure(1); clf;
figure(2); clf;
figure(3); clf;
figure(4); clf;
Kpstr = {};
for i=1:4
    % Set numerator & denominator coefficients
    Gnum = b0;       % Numerator coefficients of G
    Gden = [1 a0];   % Denominator coefficients of G
        
    % Simulate Simulink model
    r = 1000;   % Desired speed, rad/sec
    if i==1
        Kp = 0.002;
    elseif i==2
        Kp = 0.005;
    elseif i==3
        Kp = 0.01;
    else
        Kp = 0.05;
    end
    Kpstr{i} = ['Kp = ' num2str(Kp)];
    sim('MotorPControl')
    
    % Plot input voltage
    figure(1);
    plot(t,u,'Color',co(i,:));
    hold on;
    
    % Plot motor speed
    figure(2);
    plot(t,y,'Color',co(i,:));
    hold on;
    plot([0 Tf],[r r],'k:');
    
    % Plot speed error
    figure(3);
    plot(t,e,'Color',co(i,:));
    hold on;
    
    % Plot locations of closed-loop pole
    figure(4);
    p = -(a0+b0*Kp);
    plot(p,0,'x','Color',co(i,:),'MarkerSize',12);
    hold on;
end

figure(1);
xlabel('Time (sec)');
ylabel('Input u (V)');
grid on;
hold off;
axis([0 Tf 0 umax]);
set(gca, 'LooseInset', get(gca, 'TightInset'));

figure(2);
xlabel('Time (sec)');
ylabel('Speed, y (rad/sec)');
axis([0 Tf 0 1200]);
grid on;
hold off;
set(gca, 'LooseInset', get(gca, 'TightInset'));

figure(3);
xlabel('Time (sec)');
ylabel('Error, e (rad/sec)');
axis([0 Tf 0 1000]);
grid on;
hold off;
set(gca, 'LooseInset', get(gca, 'TightInset'));

figure(4)
grid
xlabel('Real');
ylabel('Imag');
legend(Kpstr{:});
grid on;
hold off;
set(gca, 'LooseInset', get(gca, 'TightInset'));

if exist('garyfyFigure','file') && exist('myprint','file')
    % Code to clean up and save figure.
    % These functions are not in the standard Matlab release.
    figure(1)
    garyfyFigure
    myprint(1,'DCPCont_NomInput')
    
    figure(2)
    garyfyFigure
    myprint(2,'DCPCont_NomOutput')
    
    figure(3)
    garyfyFigure
    myprint(3,'DCPCont_NomError')
    
    figure(4)
    garyfyFigure
    myprint(4,'DCPCont_NomPoles')
end

%% Simulations with Parameter Variations 
% Simulate step responses with coefficient values selected within the 
% range of uncertainty. Overlay the nominal motor response (nominal 
% coefficients and without saturation).


% Simulation and plot settings
ulim = inf;     % Limit on max motor voltage for simulation, V
d = 0;          % Input disturbance, V
Tf = 1;         % Final simulation time, sec
Kp = 0.01;      % Proportional Gain

% Plot step responses
figure(5); clf;
figure(6); clf;
for i=1:5
    % Simulations 1-4 correspond to +/-10% uncertainty in the
    % ODE coefficients. Simulation 5 gives the nominal response.
    if i==1
        a0 = 1.1*a0Nom;
        b0 = 1.1*b0Nom;
    elseif i==2
        a0 = 1.1*a0Nom;
        b0 = 0.9*b0Nom;
    elseif i==3
        a0 = 0.9*a0Nom;
        b0 = 1.1*b0Nom;
    elseif i==4
        a0 = 0.9*a0Nom;
        b0 = 0.9*b0Nom;
    else
        % Use nominal values and no saturation for last simulation
        a0 = a0Nom;
        b0 = b0Nom;
        ulim = inf;
    end
    Gnum = b0;       % Numerator coefficients of G
    Gden = [1 a0];   % Denominator coefficients of G
    
    % Simulate Simulink model
    r = 1000;   % Desired speed, rad/sec
    sim('MotorPControl')
    
    % Plot input voltage and  motor speed
    if i<5
        figure(5);
        plot(t,u,'r-.');
        hold on;
        
        figure(6);
        ph(i)=plot(t,y,'r-.'); 
        hold on;
    else
        figure(5);
        plot(t,u,'b');
        
        figure(6);
        ph(i)=plot(t,y,'b');
    end
    
end
figure(5);
xlabel('Time (sec)');
ylabel('Input u (V)');
grid on;
hold off;
axis([0 Tf 0 10]);
set(gca, 'LooseInset', get(gca, 'TightInset'));

figure(6);
ph(6) = plot([0 Tf],[r r],'k:');
xlabel('Time (sec)');
ylabel('Speed, y (rad/sec)');
legend([ph(5); ph(4); ph(6)],'Nominal','+/-10% Variation','Desired',...
    'Location','Best');
axis([0 Tf 0 1200]);
grid on;
hold off;
set(gca, 'LooseInset', get(gca, 'TightInset'));

if exist('garyfyFigure','file') && exist('myprint','file')
    % Code to clean up and save figure.
    % These functions are not in the standard Matlab release.
    figure(5)
    garyfyFigure
    myprint(5,'DCPCont_PVInput')    
    
    figure(6)
    garyfyFigure
    myprint(6,'DCPCont_PVOutput')    
end



%% Simulations on Model with Fast Dynamics
% The proporitional controller was designed on the first-order
% model with the dominant inertia dynamics. This model neglects the
% fast electrical dynamics.  This part of the code simulates
% the controller (with different proportional gains) on a model that
% includes the fast dynamics.

% Second-order model
rpm2radpsec = 2*pi/60;          % Conversion from rpm to rad/sec
urated = 2.4;                   % Rated Voltage, V
yNL = 17000*rpm2radpsec;        % No load speed at rated voltage, rad/sec
iNL = 270e-3;                   % No load current, Amps
R = 0.8;                        % Motor resistance, Ohms
L = 150e-6;                     % Motor inductance, H
J = 2e-6;                       % Inertia, kg*m^2
b = 0;                          % Damping friction const., N*m/(rad/sec)
Ke = (urated - R*iNL)/yNL;      % Back EMF constant, V/(rad/sec);
Kt = Ke;                        % Torque constant, N*m/Amp

G2 = tf(Kt,[J*L (J*R+b*L) (Kt*Ke+b*R)]); 

% Simulation and plot settings
ulim = inf;     % Limit on max motor voltage for simulation, V
d =  0;         % Input disturbance, V
Tf = 0.005;         % Final simulation time, sec

% Simulate with several proportional gains
figure(7); clf;
figure(8); clf;
Kpstr = {};
for i=1:6
    % Set numerator & denominator coefficients
    Gnum = Kt;                            % Numerator coefficients of G
    Gden = [J*L (J*R+b*L) (Kt*Ke+b*R)];   % Denominator coefficients of G

    % Simulate Simulink model
    r = 1000;   % Desired speed, rad/sec
    if i==1
        Kp = 0;
    elseif i==2
        Kp = 1;
    elseif i==3
        Kp = 1.5;
    elseif i==4
        Kp = 2;
    elseif i==5 
        Kp = 8;
    else
        Kp = 16;
    end
    Kpstr{i} = ['Kp = ' num2str(Kp)];
    sim('MotorPControl')
    
    % Plot motor speed
    if i>1
        figure(7);
        plot(t,y,'Color',co(i,:));
        hold on;
    end
    
    % Plot locations of closed-loop pole
    figure(8);
    p = roots([J*L (J*R+b*L) (Kt*Ke+b*R)+Kp*Gnum]);
    [wn,z] = damp(r(1));
    if i==1
        p0 = p;
        plot(real(p),imag(p),'kx','MarkerSize',12);
    else
        pf = p;
        plot(real(p),imag(p),'x','Color',co(i,:),'MarkerSize',12);
    end
    hold on;
end

figure(7);
plot([0 Tf],[r r],'k:');
xlabel('Time (sec)');
ylabel('Speed, y (rad/sec)');
legend(Kpstr{2:end},'Location','Southeast');
grid on;
hold off;
set(gca, 'LooseInset', get(gca, 'TightInset'));

figure(8)
ax = axis;
ph = plot([p0(1) p0(2)],[0 0],'k',real(pf),[ax(3) ax(4)],'k');
axis(ax);
grid
xlabel('Real');
ylabel('Imag');
legend(Kpstr{:},'Location','NorthWest');
grid on;
hold off;
set(gca, 'LooseInset', get(gca, 'TightInset'));

if exist('garyfyFigure','file') && exist('myprint','file')
    % Code to clean up and save figure.
    % These functions are not in the standard Matlab release.    
    figure(7)
    garyfyFigure
    myprint(7,'DCPCont_HFInput')
        
    figure(8)
    garyfyFigure
    set(ph,'LineWidth',1);
    myprint(8,'DCPCont_HFOutput')
end



