%% Discrete-time Signals
% This file generates plots showing an example of the continuous-time 
% measurement, discrete-time sampled measurement, discrete-time control 
% signal, and continuous-time (zero-order hold) control signal.

%% Measurement signals

% Continuous-time
Nc = 50;
tc = linspace(0,0.55,Nc);
yc = 10*tc -48*tc.^2 + 60*tc.^3;

% Sampled Discrete-time
delt = 0.1;         % Sample Time Interval, sec
td = delt:delt:0.5;      % Sample Times, sec
Nd = numel(td);
yd = 10*td -48*td.^2 + 60*td.^3;

%% Continuous time measurement
figure(1); clf;
plot(tc,yc,'b'); 
hold on;
ph = [];
for k=1:Nd;
    ph(k) = plot( [td(k) td(k)], [0 yd(k)],'r:');
end
xlabel('Time, $t$');
ylabel('Output, $y(t)$');
xlim([0 tc(end)]);
grid on;
set(gca, 'LooseInset', get(gca, 'TightInset'));
if exist('garyfyFigure','file') && exist('matlab2tikz','file');
    % Code to clean up and save figure.
    % These functions are not in the standard Matlab release.
    garyfyFigure
    set(ph,'LineWidth',2)
    
    matlab2tikz('C2D_YCont.tikz','width','\figurewidth',...
         'parseStrings',false,'extraAxisOptions',...
         'ylabel near ticks, xlabel near ticks');
end

%% Discrete time (sampled) measurement

figure(2); clf;
ph = plot(tc,yc,'b:');
hold on;
for k=1:Nd;
    plot( [td(k) td(k)], [0 yd(k)],'r');
    plot( td(k), yd(k), 'r.','MarkerSize',12);
    if k<Nd
        text(td(k)-0.03,yd(k)+0.1,['$y_' int2str(k) '$']);
    else
        text(td(k)-0.075,yd(k),['$y_' int2str(k) '$']);
    end
end
hold off;
xlabel('Time Index, $k$');
ylabel('Output, $y_k$');
set(gca,'Xtick',0:delt:1,'XTickLabel',int2str((0:5)'));
xlim([0 tc(end)]);
grid on;
set(gca, 'LooseInset', get(gca, 'TightInset'));
if exist('garyfyFigure','file') && exist('matlab2tikz','file');
    % Code to clean up and save figure.
    % These functions are not in the standard Matlab release.
    garyfyFigure
    set(ph,'LineWidth',2)
    
    matlab2tikz('C2D_YDisc.tikz','width','\figurewidth',...
         'parseStrings',false,'extraAxisOptions',...
         'ylabel near ticks, xlabel near ticks');
end

%% Difference Equation update equation
ed = zeros(Nd,1);
ud = zeros(Nd,1);
rd = 1;
for k=1:Nd
    ed(k) = rd-yd(k);
    if k==1
        ud(k) = 5*ed(k);
    else
        ud(k) = ud(k-1) + 5*ed(k) - 4.9*ed(k-1);
    end
end

%% Discrete time control

figure(3); clf;
ph(1) = plot([0 td(1)],[0 0],'b:');
hold on;
for k=1:Nd;
    plot( [td(k) td(k)], [0 ud(k)],'r');
    plot( td(k), ud(k), 'r.','MarkerSize',12);
    if k<Nd
        ph(k+1)=plot([td(k) td(k+1)],[ud(k) ud(k)],'b:');
    else
        ph(k+1)=plot([td(k) tc(end)],[ud(k) ud(k)],'b:');
    end        
    text(td(k)-0.04,ud(k)+0.5,['$u_' int2str(k) '$']);
end
hold off;
xlabel('Time Index, $k$');
ylabel('Control, $u_k$');
set(gca,'Xtick',0:delt:1,'XTickLabel',int2str((0:5)'));
xlim([0 tc(end)]);
ylim([-0.05 5.5]);
grid on;
set(gca, 'LooseInset', get(gca, 'TightInset'));
if exist('garyfyFigure','file') && exist('matlab2tikz','file');
    % Code to clean up and save figure.
    % These functions are not in the standard Matlab release.
    garyfyFigure
    set(ph,'LineWidth',2)
    
    matlab2tikz('C2D_UDisc.tikz','width','\figurewidth',...
         'parseStrings',false,'extraAxisOptions',...
         'ylabel near ticks, xlabel near ticks');
end

%% Continuous time control

figure(4); clf;
plot([0 td(1)],[0 0],'b');
hold on;
ph = [];
for k=1:Nd;
    ph(k)=plot( [td(k) td(k)], [0 ud(k)],'r:');
    if k<Nd
        plot([td(k) td(k+1)],[ud(k) ud(k)],'b');
    else
        plot([td(k) tc(end)],[ud(k) ud(k)],'b');
    end        
end
hold off;
xlabel('Time, $t$');
ylabel('Control, $u(t)$');
xlim([0 tc(end)]);
ylim([-0.05 5.5]);
grid on;
set(gca, 'LooseInset', get(gca, 'TightInset'));
set(gca,'XtickLabelMode','manual')
if exist('garyfyFigure','file') && exist('matlab2tikz','file');
    % Code to clean up and save figure.
    % These functions are not in the standard Matlab release.
    garyfyFigure
    set(ph,'LineWidth',2)
    
    matlab2tikz('C2D_UCont.tikz','width','\figurewidth',...
         'parseStrings',false,'extraAxisOptions',...
         'ylabel near ticks, xlabel near ticks');
end
